//+------------------------------------------------------------------+
//|                                              Moving Averages.mq5 |
//|              Copyright Copyright 2010, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"

#include <ExpertAdvisor.mqh>

input double Risk      = 0.1; // 
input int    SL        =  30; //  
input int    TP        =  90; //  
input int    TS        =  30; //   
input int    pMA       =   5; //   
input int    pMA75     =  75; //   
input int    pMA85     =  85; //   

//---
class CMyEA : public CExpertAdvisor
  {
protected:
   double            m_risk;          //  
   int               m_sl;            // 
   int               m_tp;            // 
   int               m_ts;            // 
   int               m_hourStart;     //   
   int               m_hourEnd;       //   
   int               m_hma5;          //  
   int               m_hma75;         //  
   int               m_hma85;         //   
   int               m_hmacd;         //  CD        
public:
   void              CMyEA();
   void             ~CMyEA();
   virtual bool      Init(string smb,ENUM_TIMEFRAMES tf); // 
   virtual bool      Main();                              //  
   virtual void      OpenPosition(long dir);              //    
   virtual void      ClosePosition(long dir);             //    
   virtual long      CheckSignal(bool bEntry);            //  
  };
//------------------------------------------------------------------	CMyEA
void CMyEA::CMyEA() { }
//------------------------------------------------------------------	~CMyEA
void CMyEA::~CMyEA()
  {
   IndicatorRelease(m_hma5);  //   
   IndicatorRelease(m_hma75); //   
   IndicatorRelease(m_hma85); //   
   IndicatorRelease(m_hmacd); //   CD         
  }
//------------------------------------------------------------------	Init
bool CMyEA::Init(string smb,ENUM_TIMEFRAMES tf)
  {
   if(!CExpertAdvisor::Init(0,smb,tf)) return(false);  //   

   m_risk=Risk; m_tp=TP; m_sl=SL; m_ts=TS;          //  
   m_hma5=iMA(m_smb,m_tf,5,0,MODE_EMA,PRICE_CLOSE); //   
   m_hma75=iMA(m_smb,m_tf,75,0,MODE_LWMA,PRICE_LOW);//   
   m_hma85=iMA(m_smb,m_tf,85,0,MODE_LWMA,PRICE_LOW);//    
   m_hmacd=iMACD(m_smb,m_tf,15,26,1,PRICE_CLOSE);   //   CD
   if(m_hma5==INVALID_HANDLE  || 
      m_hma75==INVALID_HANDLE ||
      m_hma85==INVALID_HANDLE ||
      m_hmacd==INVALID_HANDLE ) return(false);      //    
   m_bInit=true; return(true);                      // " "
  }
//------------------------------------------------------------------	Main
bool CMyEA::Main() //  
  {
   if(!CExpertAdvisor::Main()) return(false); //    

   if(Bars(m_smb,m_tf)<=85) return(false);    //   

   if(!CheckNewBar()) return(true);           //   

                                              //   
   long dir;
   dir=ORDER_TYPE_BUY;
   OpenPosition(dir); ClosePosition(dir); TrailingPosition(dir,m_ts);
   dir=ORDER_TYPE_SELL;
   OpenPosition(dir); ClosePosition(dir); TrailingPosition(dir,m_ts);

   return(true);
  }
//------------------------------------------------------------------	OpenPos
void CMyEA::OpenPosition(long dir)
  {
//   ,  
   if(PositionSelect(m_smb)) return;
//      
   if(dir!=CheckSignal(true)) return;
   double lot=CountLotByRisk(m_sl,m_risk,0);
//    
   if(lot<=0) return;
//  
   DealOpen(dir,lot,m_sl,m_tp);
  }
//------------------------------------------------------------------	ClosePos
void CMyEA::ClosePosition(long dir)
  {
//   ,  
   if(!PositionSelect(m_smb)) return;
//     
   if(dir!=PositionGetInteger(POSITION_TYPE)) return;
//        
   if(dir!=CheckSignal(false)) return;
//  
   m_trade.PositionClose(m_smb,1);
  }
//------------------------------------------------------------------	CheckSignal
long CMyEA::CheckSignal(bool bEntry)
  {

   double ma[1],ma75[1],ma85[1],macd[1];
   if(CopyBuffer(m_hma5,0,0,1,ma)!=1 || CopyBuffer(m_hma75,0,0,1,ma75)!=1 || CopyBuffer(m_hma85,0,0,1,ma85)!=1 || CopyBuffer(m_hmacd,0,0,1,macd)!=1)
     { Print("CopyBuffer MA -  "); return(WRONG_VALUE); }

   if(ma[0]>ma75[0] && ma[0]>ma75[0] && macd[0]>0)
      return(bEntry ? ORDER_TYPE_BUY:ORDER_TYPE_SELL); //                   
   if(ma[0]<ma75[0] && ma[0]<ma75[0] && macd[0]<0)
      return(bEntry ? ORDER_TYPE_SELL:ORDER_TYPE_BUY); //                     
   return(WRONG_VALUE); //   
  }

CMyEA ea; //  
//------------------------------------------------------------------	OnInit
int OnInit()
  {
   ea.Init(Symbol(),Period()); //  
   return(0);
  }
//------------------------------------------------------------------	OnDeinit
void OnDeinit(const int reason) { }
//------------------------------------------------------------------	OnTick
void OnTick()
  {
   ea.Main(); //   
  }
//+------------------------------------------------------------------+
